import { __awaiter, __decorate, __param, __rest } from "tslib";
import { observable } from '@microsoft/fast-element';
import { DI } from '@microsoft/fast-foundation';
import { combineLatest, from, timer } from 'rxjs';
import { switchMap, takeWhile } from 'rxjs/operators';
import { Auth } from '../auth';
import { Connect } from '../connect';
import { extractFieldDefinitions, toFieldMetadata } from '../metadata/metadata.utils';
import { GenesisResources } from '../resources';
import { logger } from '../utils';
import { DatasourceConfig } from './datasource.config';
import { DatasourceStatusChanged, ResourceType } from './datasource.types';
/**
 * The default implementation for the Datasource interface.
 * @public
 */
let DefaultDatasource = class DefaultDatasource {
    /** {@inheritDoc Datasource.initialized} */
    get initialized() {
        var _a;
        return !!((_a = this.status) === null || _a === void 0 ? void 0 : _a.isInitialized);
    }
    constructor(connect, auth, resources, config) {
        this.connect = connect;
        this.auth = auth;
        this.resources = resources;
        this.config = config;
        /** {@inheritDoc DatasourceMetadata.fetchMetadataRequired} */
        this.fetchMetadataRequired = true;
        /** {@inheritDoc DatasourceMetadata.startStream} */
        this.startStreamRequired = true;
        /** {@inheritDoc DatasourceMetadata.originalFieldDef} */
        this.originalFieldDef = [];
        /** {@inheritDoc DatasourceMetadata.fieldMetadata} */
        this.fieldMetadata = [];
        /** {@inheritDoc DatasourceMetadata.requestFields} */
        this.requestFields = [];
        /** {@inheritDoc DatasourceMetadata.customRequestFields} */
        this.customRequestFields = [];
        /** {@inheritDoc DatasourceMetadata.availableIndexes} */
        this.availableIndexes = [];
        this.readOnly = true;
    }
    /**
     * @deprecated - Please use {@link (GenesisResources:interface).isValidResource} instead.
     */
    validResourceName(resourceName) {
        logger.deprecated('Datasource.validResourceName', 'Use `GenesisResources.isValidResource` instead.');
        if (!resourceName)
            return false;
        if (!resourceName.trim().length)
            return false;
        // additional checks? regex? or maybe we can ask the server if it's a valid resource in the future
        return true;
    }
    init(options, fetchMeta = this.fetchMetadataRequired, startStream = this.startStreamRequired) {
        return __awaiter(this, void 0, void 0, function* () {
            this.fetchMetadataRequired = fetchMeta;
            this.options = Object.assign(Object.assign({}, this.config.options), options);
            const resourceName = this.options.resourceName;
            const connected = this.connect.isConnected || (yield this.connect.connect());
            if (!connected) {
                logger.error(`Unable to connect to ${this.connect.host} to get ${resourceName} datasource`);
                this.reportStatus();
                return false;
            }
            const isValidResource = yield this.resources.isValidResource(resourceName);
            if (!isValidResource) {
                logger.error(`${resourceName} datasource does not exist on ${this.connect.host}`);
                this.reportStatus();
                return false;
            }
            this.resourceType = yield this.resources.getResourceTypeFor(resourceName);
            if (!this.resourceType) {
                logger.error(`Unable to get the resource type for ${resourceName} datasource from ${this.connect.host}`);
                this.reportStatus();
                return false;
            }
            logger.debug(`Initializing ${resourceName} datasource from ${this.connect.host}`);
            if (!this.options.isSnapshot && startStream) {
                this.stream = yield this.createStream();
            }
            if (this.fetchMetadataRequired) {
                yield this.fetchAndApplyMetadata(this.options.resourceName);
            }
            logger.debug(`${resourceName} datasource initialization ${this.initialized ? `was successful.` : 'has failed.'}`);
            this.reportStatus();
            return this.initialized;
        });
    }
    startStream() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.options.isSnapshot && this.stream === undefined) {
                this.stream = yield this.createStream();
            }
        });
    }
    /** {@inheritDoc Datasource.destroy} */
    destroy() {
        var _a;
        logger.debug('Datasource.destroy', (_a = this.options) === null || _a === void 0 ? void 0 : _a.resourceName);
        this.status = undefined;
        this.fetchMetadataRequired = undefined;
        this.options = {};
        this.resourceType = undefined;
        this.availableIndexes = [];
        this.originalFieldDef = [];
        this.fieldMetadata = [];
        this.requestFields = [];
        this.stream = undefined;
    }
    get allParams() {
        var _a;
        let ORDER_BY;
        if (this.options.orderBy) {
            if (((_a = this.availableIndexes) === null || _a === void 0 ? void 0 : _a.findIndex((i) => i.NAME === this.options.orderBy)) > -1) {
                ORDER_BY = this.options.orderBy;
            }
            else {
                const validIndexes = this.availableIndexes.map((index) => index.NAME);
                logger.warn('Specified [orderBy] index is not valid and will not be used. See https://learn.genesis.global/docs/database/data-types/index-entities/ - Available indexes:', validIndexes);
            }
        }
        const params = {
            // Common between DATASERVER and REQUEST_SERVER (REQ-REP)
            CRITERIA_MATCH: this.options.criteria,
            MAX_ROWS: this.options.maxRows,
            VIEW_NUMBER: this.options.viewNumber,
            // DATASERVER only
            FIELDS: this.options.fields,
            MAX_VIEW: this.options.maxView,
            MOVING_VIEW: this.options.movingView,
            ORDER_BY,
            REVERSE: this.options.reverse,
            // REQUEST_SERVER only
            DISABLE_POLLING: this.options.disablePolling,
            POLLING_INTERVAL: this.options.pollingInterval,
            REQUEST: this.options.request,
        };
        logger.debug('Getting configured params:', params);
        return params;
    }
    get dataserverParams() {
        return {
            CRITERIA_MATCH: this.allParams.CRITERIA_MATCH,
            MAX_ROWS: this.allParams.MAX_ROWS,
            VIEW_NUMBER: this.allParams.VIEW_NUMBER,
            FIELDS: this.allParams.FIELDS,
            MAX_VIEW: this.allParams.MAX_VIEW,
            MOVING_VIEW: this.allParams.MOVING_VIEW,
            ORDER_BY: this.allParams.ORDER_BY,
            REVERSE: this.allParams.REVERSE,
        };
    }
    get requestParams() {
        var _a;
        const _b = this.allParams, { REQUEST } = _b, DETAILS = __rest(_b, ["REQUEST"]);
        const reqParams = {
            DETAILS: {
                CRITERIA_MATCH: !this.options.requestAutoSetup ? DETAILS.CRITERIA_MATCH : undefined,
                MAX_ROWS: DETAILS.MAX_ROWS,
                VIEW_NUMBER: DETAILS.VIEW_NUMBER,
            },
            REQUEST,
        };
        if (!reqParams.REQUEST) {
            if (this.options.requestAutoSetup) {
                reqParams.REQUEST = Object.assign(Object.assign({}, Object.fromEntries(this.requestFields.map((rf) => [rf.NAME, '*']))), (((_a = this === null || this === void 0 ? void 0 : this.options) === null || _a === void 0 ? void 0 : _a.criteria) ? this.config.criteriaToFields(this.options.criteria) : {}));
            }
            else {
                reqParams.REQUEST = {};
            }
        }
        return reqParams;
    }
    snapshot(overrideParams) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug(`Requesting snapshot for ${this.options.resourceName} (${this.resourceType}).`);
            switch (this.resourceType) {
                case ResourceType.DATASERVER:
                    return this.connect.snapshot(this.options.resourceName, Object.assign(Object.assign({}, this.dataserverParams), overrideParams));
                case ResourceType.REQUEST_SERVER: {
                    return this.connect.request(this.options.resourceName, Object.assign(Object.assign({}, this.requestParams), overrideParams));
                }
                default:
                    throw new Error(`Snapshot not supported for selected resource type: ${this.resourceType}`);
            }
        });
    }
    snapshotFiltered(rowId = 'ROW_REF') {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.snapshot();
            if (Array.isArray(result)) {
                return result;
            }
            else if (result.REPLY) {
                return result.REPLY;
            }
            else if (result.ROW) {
                return this.config.dataServerResultFilter(result, rowId).inserts;
            }
        });
    }
    createStream() {
        return __awaiter(this, void 0, void 0, function* () {
            if (![ResourceType.DATASERVER, ResourceType.REQUEST_SERVER].includes(this.resourceType)) {
                throw new Error(`Streaming not supported for selected resource type: ${this.resourceType}`);
            }
            if (this.resourceType === ResourceType.DATASERVER) {
                return this.createDataserverStream();
            }
            logger.debug(`Creating fake stream by polling ${this.options.resourceName} REQUEST_SERVER`);
            return this.createReqRepStream();
        });
    }
    createDataserverStream() {
        return __awaiter(this, void 0, void 0, function* () {
            const onMessage = (message) => { };
            const onError = (error) => logger.error(error);
            return this.connect.stream(this.options.resourceName, onMessage, onError, this.dataserverParams);
        });
    }
    createReqRepStream() {
        return timer(0, this.allParams.DISABLE_POLLING ? undefined : this.allParams.POLLING_INTERVAL).pipe(switchMap(() => {
            // Combine the isConnected status with the isLoggedIn status
            return combineLatest([this.connect.isConnected$, this.auth.isLoggedIn$]).pipe(
            // Use takeWhile to continue polling until isConnected becomes false or isLoggedIn becomes false
            takeWhile(() => this.connect.isConnected && this.auth.isLoggedIn), // Use switchMap to get the snapshot data
            switchMap(() => from(this.snapshot())));
        }));
    }
    fetchAndApplyMetadata(resourceName) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                logger.debug(`Requesting metadata for ${this.options.resourceName} Datasource`);
                const meta = yield this.connect.getMetadata(this.options.resourceName);
                if (meta) {
                    this.availableIndexes = (_a = meta.INDEXES) !== null && _a !== void 0 ? _a : [];
                    this.originalFieldDef = extractFieldDefinitions(meta);
                    this.fieldMetadata = toFieldMetadata(this.originalFieldDef);
                    this.requestFields = meta.REQUEST_FIELD || [];
                    this.customRequestFields = meta.FIELD || [];
                    this.fetchMetadataRequired = false;
                }
                this.status = this.normaliseStatus({
                    isInitialized: true,
                    metadataRequired: true,
                    hasMetadata: ((_b = this.fieldMetadata) === null || _b === void 0 ? void 0 : _b.length) > 0,
                });
            }
            catch (e) {
                logger.error(`Metadata Fetch failed for ${resourceName}:`, e);
                this.status = this.normaliseStatus({
                    isInitialized: false,
                    metadataRequired: true,
                    hasMetadata: false,
                });
            }
        });
    }
    /**
     * @internal
     */
    reportStatus(detail = {}) {
        var _a;
        const eventEmitter = (_a = this.config.eventEmitter) !== null && _a !== void 0 ? _a : document;
        this.status = this.normaliseStatus(detail);
        logger.debug(`Dispatching '${DatasourceStatusChanged}'`, this.status);
        eventEmitter === null || eventEmitter === void 0 ? void 0 : eventEmitter.dispatchEvent(new CustomEvent(DatasourceStatusChanged, {
            detail: this.status,
            bubbles: eventEmitter !== document,
            composed: true,
        }));
    }
    /**
     * @internal
     */
    normaliseStatus(status) {
        var _a, _b, _c, _d, _e;
        const isInitialized = (_a = status.isInitialized) !== null && _a !== void 0 ? _a : this.initialized;
        const metadataRequired = (_b = status.metadataRequired) !== null && _b !== void 0 ? _b : this.fetchMetadataRequired;
        const hasMetadata = (_c = status.hasMetadata) !== null && _c !== void 0 ? _c : ((_d = this.fieldMetadata) === null || _d === void 0 ? void 0 : _d.length) > 0;
        return {
            name: (_e = this.options) === null || _e === void 0 ? void 0 : _e.resourceName,
            type: this.resourceType,
            isInitialized,
            metadataRequired,
            hasMetadata,
        };
    }
};
__decorate([
    observable
], DefaultDatasource.prototype, "status", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "fetchMetadataRequired", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "startStreamRequired", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "originalFieldDef", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "fieldMetadata", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "requestFields", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "customRequestFields", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "availableIndexes", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "readOnly", void 0);
__decorate([
    observable
], DefaultDatasource.prototype, "stream", void 0);
DefaultDatasource = __decorate([
    __param(0, Connect),
    __param(1, Auth),
    __param(2, GenesisResources),
    __param(3, DatasourceConfig)
], DefaultDatasource);
export { DefaultDatasource };
/**
 * The DI token for the Datasource interface.
 * @internal
 */
export const Datasource = DI.createInterface((x) => x.transient(DefaultDatasource));
/**
 * Get Datasource from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { FilteredDataServerResult, getDatasource, SocketObservable } from '@genesislcap/foundation-comms';
 * ...
 * private ds = getDatasource();
 * private dsSub: SocketObservable<FilteredDataServerResult>;
 * ...
 * const dsInit = await this.ds.init({
 *   criteria: '',
 *   isSnapshot: false,
 *   resourceName: 'ALL_POSITIONS',
 *   reverse: false,
 *   maxRows: 250,
 *   maxView: 1000,
 * });
 * if (!dsInit) {
 *   logger.debug(`Failed to initialise 'ALL_POSITIONS' datasource`);
 *   return;
 * }
 * this.dsSub = this.ds.stream.subscribe((result) => { ... });
 * ```
 *
 * @public
 */
export function getDatasource() {
    return DI.getOrCreateDOMContainer().get(Datasource);
}
